#!/bin/bash

set -e

source config
LOG=${LOG_DIR}/qual_driver.log
> $LOG # clear the log

DG_LOG=${LOG_DIR}/qual_datagen.log
> $DG_LOG # clear the log

LD_LOG=${LOG_DIR}/qual_load.log
> $LD_LOG # clear the log

PARQUET_DB=tpcds_qual_parquet
TEXT_DB=tpcds_qual_text

function gen_data() {
  # Copy from offical toolset, apply patches, build dsqgen and dsdgen
  rm -fr ${BUILD_DIR}/${TOOLS_VERSION}; cp -r $bin/${TOOLS_VERSION} ${BUILD_DIR}

  cd ${BUILD_DIR}/${TOOLS_VERSION}/tools; make clean; make dsdgen
  log info `./dsdgen -release 2>&1`

  # generate qualification dataset and load to text tables and qualification db tables
  mkdir -p ${BUILD_DIR}/qualification_data
  rm -fr ${BUILD_DIR}/qualification_data/*
  ./dsdgen -scale 1 -dir ${BUILD_DIR}/qualification_data
}

function load_data() {
  cd $bin
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_qual_text_tables.sql --hivevar bin=${bin} --hivevar QUAL_TEXT_DB=$TEXT_DB

  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_store_sales.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_store_returns.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_catalog_sales.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_catalog_returns.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_web_sales.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_web_returns.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_inventory.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/create_dim_tables.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}

  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/analyze_store_sales.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/analyze_catalog_sales.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/analyze_web_sales.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/analyze_store_returns.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/analyze_catalog_returns.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/analyze_web_returns.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/analyze_inventory.sql --hivevar DB=$PARQUET_DB --hivevar SOURCE=$TEXT_DB --hivevar WAREHOUSE_DIR=${WAREHOUSE_DIR}
}

function table_schema() {
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/desc_tables.sql -hivevar DB=$PARQUET_DB 2>&1
}

function row_count() {
  beeline -u jdbc:hive2://localhost:${PORT}/ -f ./sqls/count_tables.sql -hivevar DB=$PARQUET_DB 2>&1
}

function run_queries() {
  # run queries one by one
  mkdir -p $LOG_DIR/qualification_result
  for i in $@; do beeline --nullemptystring=true --outputformat=tsv2 -u jdbc:hive2://localhost:${PORT}/$PARQUET_DB -f ${bin}/qualification_queries/query$i.sql > $LOG_DIR/qualification_result/query$i.out; done
}

mkdir -p ${BUILD_DIR}
echo "use $PARQUET_DB;" > ${BUILD_DIR}/db.sql

log info "Generating data ..." | tee -a $DG_LOG | tee -a $LOG
gen_data >> $DG_LOG 2>&1

log info "Loading data ..." | tee -a $LD_LOG | tee -a $LOG
load_data >> $LD_LOG 2>&1

log info "Get table schema ..." | tee -a $LOG
table_schema > $LOG_DIR/qual_table_schema.out

log info "Count tables ..." | tee -a $LOG
row_count > $LOG_DIR/qual_row_count.out

log info "Running sqls ..." | tee -a $LOG
run_queries `seq 99` >> $LOG 2>&1
